@@ -12,12 +12,9 @@ if sys.argv[1] == 'run_all_tests':
     try:
         for arg in sys.argv[4:]:
             if arg[:2] == "--":
-                print(arg[2:])
                 arg_split = arg[2:].split('=')
-                print(arg_split)
                 arg_key = arg_split[0]
                 arg_value = arg_split[1]
-                print(f"{arg_key} is {arg_value}")
                 if arg_key.lower() == 'liveport':
                     live_port = int(arg_value)
                 if arg_key.lower() == 'histport':
