@@ -30,21 +30,17 @@ class TDInvalidRequestError(Exception):
 
 class LiveClient(WebSocketApp):
 
-    def __init__(self, parent_app, url, on_open=None, *args):
-        if on_open is None:
-            on_open = self.on_open_func
-        WebSocketApp.__init__(self, url, on_open=on_open, on_message=self.on_msg_func, *args)
+    def __init__(self, parent_app, url, *args):
+        WebSocketApp.__init__(self, url, on_error=self.ws_error, on_message=self.on_msg_func, *args)
         self.segments = []
         self.max_symbols = 0
         self.remaining_symbols = 0
         self.valid_until = ''
         self.contract_mapping = {}
         self.subscription_type = ''
+        self.sock.settimeout(300)
         self.parent_app = parent_app
 
-    def on_open_func(self):
-        pass
-
     def on_msg_func(self, message):
         msg = json.loads(message)
         if 'message' in msg.keys():
@@ -81,16 +77,17 @@ class LiveClient(WebSocketApp):
             self.contract_mapping[int(contract_details[1])] = symbol = contract_details[0]
             for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
                 self.parent_app.touchline_data[ticker_id].symbol = symbol
-                self.parent_app.touchline_data[ticker_id].truedata_id = int(contract_details[1])
-                self.parent_app.touchline_data[ticker_id].open = self.parent_app.live_data[ticker_id].day_open = float(contract_details[2])
-                self.parent_app.touchline_data[ticker_id].high = self.parent_app.live_data[ticker_id].day_high = float(contract_details[3])
-                self.parent_app.touchline_data[ticker_id].low = self.parent_app.live_data[ticker_id].day_low = float(contract_details[4])
+                self.parent_app.touchline_data[ticker_id].symbol_id = int(contract_details[1])
+                self.parent_app.touchline_data[ticker_id].open = float(contract_details[2])
+                self.parent_app.touchline_data[ticker_id].high = float(contract_details[3])
+                self.parent_app.touchline_data[ticker_id].low = float(contract_details[4])
                 self.parent_app.touchline_data[ticker_id].ltp = float(contract_details[5])
-                self.parent_app.touchline_data[ticker_id].prev_close = self.parent_app.live_data[ticker_id].prev_day_close = float(contract_details[6])
+                self.parent_app.touchline_data[ticker_id].prev_close = float(contract_details[6])
                 self.parent_app.touchline_data[ticker_id].ttq = int(contract_details[7])
                 self.parent_app.touchline_data[ticker_id].oi = int(contract_details[8])
-                self.parent_app.touchline_data[ticker_id].prev_oi = self.parent_app.live_data[ticker_id].prev_day_oi = int(contract_details[9])
+                self.parent_app.touchline_data[ticker_id].prev_oi = int(contract_details[9])
                 self.parent_app.touchline_data[ticker_id].turnover = float(contract_details[10])
+                self.parent_app.live_data[ticker_id].populate_using_touchline(self.parent_app.touchline_data[ticker_id])
 
     def handle_trade_data(self, trade_tick):
         try:
@@ -113,18 +110,7 @@ class LiveClient(WebSocketApp):
                         self.parent_app.live_data[ticker_id].day_low = self.parent_app.touchline_data[ticker_id].low = ltp
                 self.parent_app.live_data[ticker_id].tick_seq = int(trade_tick[8])
                 # Calculating addn data
-                try:
-                    self.parent_app.live_data[ticker_id].oi_change = self.parent_app.live_data[ticker_id].oi - self.parent_app.live_data[ticker_id].prev_day_oi
-                    self.parent_app.live_data[ticker_id].oi_change_perc = self.parent_app.live_data[ticker_id].oi_change * 100 / self.parent_app.live_data[ticker_id].prev_day_oi
-                except ZeroDivisionError:
-                    self.parent_app.live_data[ticker_id].oi_change = 0
-                    self.parent_app.live_data[ticker_id].oi_change_perc = 0
-                try:
-                    self.parent_app.live_data[ticker_id].change = self.parent_app.live_data[ticker_id].ltp - self.parent_app.live_data[ticker_id].prev_day_close
-                    self.parent_app.live_data[ticker_id].change_perc = self.parent_app.live_data[ticker_id].change * 100 / self.parent_app.live_data[ticker_id].prev_day_close
-                except ZeroDivisionError:
-                    self.parent_app.live_data[ticker_id].change = 0
-                    self.parent_app.live_data[ticker_id].change_perc = 0
+                self.parent_app.live_data[ticker_id].calculate_additional_data()
                 try:
                     self.parent_app.live_data[ticker_id].best_bid_price = float(trade_tick[9])
                     self.parent_app.live_data[ticker_id].best_bid_qty = int(trade_tick[10])
@@ -136,7 +122,7 @@ class LiveClient(WebSocketApp):
                     del self.parent_app.live_data[ticker_id].best_ask_price
                     del self.parent_app.live_data[ticker_id].best_ask_qty
         except KeyError:
-            print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
+            print(f'{Style.BRIGHT}{Fore.RED}This symbol(tick) is not tied to any req_id...{Style.RESET_ALL}')
 
     def handle_bid_ask_data(self, bidask_tick):
         try:
@@ -149,7 +135,7 @@ class LiveClient(WebSocketApp):
                 self.parent_app.live_data[ticker_id].best_ask_price = float(bidask_tick[4])
                 self.parent_app.live_data[ticker_id].best_ask_qty = int(bidask_tick[5])
         except KeyError:
-            print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
+            print(f'{Style.BRIGHT}{Fore.RED}This symbol(bid-ask) is not tied to any req_id...{Style.RESET_ALL}')
 
     def handle_bar_data(self, bar_data):
         try:
@@ -170,20 +156,12 @@ class LiveClient(WebSocketApp):
                 self.parent_app.live_data[ticker_id].volume = float(bar_data[6])
                 self.parent_app.live_data[ticker_id].oi = float(bar_data[7])
                 # Calculating addn data
-                try:
-                    self.parent_app.live_data[ticker_id].oi_change = self.parent_app.live_data[ticker_id].oi - self.parent_app.live_data[ticker_id].prev_day_oi
-                    self.parent_app.live_data[ticker_id].oi_change_perc = self.parent_app.live_data[ticker_id].oi_change * 100 / self.parent_app.live_data[ticker_id].prev_day_oi
-                except ZeroDivisionError:
-                    self.parent_app.live_data[ticker_id].oi_change = 0
-                    self.parent_app.live_data[ticker_id].oi_change_perc = 0
-                try:
-                    self.parent_app.live_data[ticker_id].change = self.parent_app.live_data[ticker_id].close - self.parent_app.live_data[ticker_id].prev_day_close
-                    self.parent_app.live_data[ticker_id].change_perc = self.parent_app.live_data[ticker_id].change * 100 / self.parent_app.live_data[ticker_id].prev_day_close
-                except ZeroDivisionError:
-                    self.parent_app.live_data[ticker_id].change = 0
-                    self.parent_app.live_data[ticker_id].change_perc = 0
+                self.parent_app.live_data[ticker_id].calculate_additional_data()
         except KeyError:
-            print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
+            print(f'{Style.BRIGHT}{Fore.RED}This symbol(bar) is not tied to any req_id...{Style.RESET_ALL}')
+
+    def ws_error(self, error):
+        print(error)
 
 
 class HistoricalWebsocket:
@@ -205,9 +183,9 @@ class HistoricalWebsocket:
             print(f"{Style.BRIGHT}{Fore.RED}Failed to connect with error message = {welcome_msg['message']}{Style.RESET_ALL}")
             self.hist_socket.close()
 
-    def get_hist_bar_data(self, contract, query_time, start_time, bar_size):
-        # print(f'{{"method": "gethistory", "interval": "{bar_size}", "symbol": "{contract}", "from": "{start_time}", "to": "{query_time}"}}')
-        self.hist_socket.send(f'{{"method": "gethistory", "interval": "{bar_size}", "symbol": "{contract}", "from": "{start_time}", "to": "{query_time}"}}')
+    def get_hist_bar_data(self, contract, end_time, start_time, bar_size):
+        # print(f'{{"method": "gethistory", "interval": "{bar_size}", "symbol": "{contract}", "from": "{start_time}", "to": "{end_time}"}}')
+        self.hist_socket.send(f'{{"method": "gethistory", "interval": "{bar_size}", "symbol": "{contract}", "from": "{start_time}", "to": "{end_time}"}}')
         raw_hist_data = self.hist_socket.recv()
         hist_data = json.loads(raw_hist_data)['data']
         hist_data = self.hist_bar_data_to_dict_list(hist_data, contract)
@@ -232,8 +210,8 @@ class HistoricalWebsocket:
             count = count + 1
         return data_list
 
-    def get_hist_tick_data(self, contract, query_time, start_time):
-        self.hist_socket.send(f'{{"method": "gethistory", "interval": "tick", "symbol": "{contract}", "from": "{start_time}", "to": "{query_time}"}}')
+    def get_hist_tick_data(self, contract, end_time, start_time):
+        self.hist_socket.send(f'{{"method": "gethistory", "interval": "tick", "symbol": "{contract}", "from": "{start_time}", "to": "{end_time}"}}')
         raw_hist_data = self.hist_socket.recv()
         hist_data = json.loads(raw_hist_data)['data']
         hist_data = self.hist_tick_data_to_dict_list(hist_data, contract)
@@ -329,68 +307,74 @@ class TD:
     def get_historic_data(self, contract,
                           ticker_id=DEFAULT_HISTORIC_DATA_ID,
                           query_time=None,
+                          end_time=None,
                           duration=None,
                           start_time=None,
                           bar_size="1 min"):
         global DEFAULT_HISTORIC_DATA_ID
+        if query_time is not None:
+            print(f"{Style.BRIGHT}{Fore.YELLOW}DEPRECATION WARNING: We are replacing query_time with end_time based on feedback from clients...\n"
+                  f"As of now, you can use both, but, preference will be given to end_time...{Style.RESET_ALL}")
+            if end_time is None:
+                end_time = query_time
         if (duration is None and start_time is None) or (duration is not None and start_time is not None):
             # print(f'Using duration over start time due to ambiguity...')
             return self.get_historical_data_from_duration(contract=contract,
                                                           ticker_id=ticker_id,
-                                                          query_time=query_time,
+                                                          end_time=end_time,
                                                           duration=duration,
                                                           bar_size=bar_size)
         if duration is not None:
             return self.get_historical_data_from_duration(contract=contract,
                                                           ticker_id=ticker_id,
-                                                          query_time=query_time,
+                                                          end_time=end_time,
                                                           duration=duration,
                                                           bar_size=bar_size)
         elif start_time is not None:
             return self.get_historical_data_from_start_time(contract=contract,
                                                             ticker_id=ticker_id,
-                                                            query_time=query_time,
+                                                            end_time=end_time,
                                                             start_time=start_time,
                                                             bar_size=bar_size)
 
     def get_historical_data_from_duration(self, contract,
                                           ticker_id=DEFAULT_HISTORIC_DATA_ID,
-                                          query_time=None,
+                                          end_time=None,
                                           duration=None,
                                           bar_size="1 min"):
         global DEFAULT_HISTORIC_DATA_ID
         if duration is None:
             duration = "1 D"
-        if query_time is None:
-            query_time = datetime.today()
+        if end_time is None:
+            end_time = datetime.today()
         else:
-            assert type(query_time) == datetime
-        start_time = self.truedata_duration_map(duration, query_time)
-        query_time = query_time.strftime('%Y-%m-%d') + 'T23:59:59'
+            assert type(end_time) == datetime
+        start_time = self.truedata_duration_map(duration, end_time)
+        end_time = end_time.strftime('%Y-%m-%d') + 'T23:59:59'
         start_time = start_time.strftime('%Y-%m-%d') + 'T00:00:00'
 
         if bar_size == 'tick':
-            hist_data = self.historical_websocket.get_hist_tick_data(contract, query_time, start_time)
+            hist_data = self.historical_websocket.get_hist_tick_data(contract, end_time, start_time)
         else:
             bar_size = bar_size.replace(' ', '')
             if bar_size[-1] == 's':
                 bar_size = bar_size[:-1]
-            hist_data = self.historical_websocket.get_hist_bar_data(contract, query_time, start_time, bar_size)
+            hist_data = self.historical_websocket.get_hist_bar_data(contract, end_time, start_time, bar_size)
         DEFAULT_HISTORIC_DATA_ID = DEFAULT_HISTORIC_DATA_ID + 1
         self.hist_data[ticker_id] = hist_data
         return hist_data
 
     def get_historical_data_from_start_time(self, contract,
                                             ticker_id=DEFAULT_HISTORIC_DATA_ID,
-                                            query_time=None,
+                                            end_time=None,
                                             start_time=None,
                                             bar_size="1 min"):
         global DEFAULT_HISTORIC_DATA_ID
-        if query_time is None:
-            query_time = datetime.today()
-            query_time = query_time.strftime('%Y-%m-%d') + 'T23:59:59'
+        if end_time is None:
+            end_time = datetime.today()
+            end_time = end_time.strftime('%Y-%m-%d') + 'T23:59:59'
         else:
-            assert type(query_time) == datetime
+            assert type(end_time) == datetime
         if start_time is None:
             start_time = datetime.today()
             start_time = start_time.strftime('%Y-%m-%d') + 'T00:00:00'
@@ -398,12 +382,12 @@ class TD:
             assert type(start_time) == datetime
 
         if bar_size == 'tick':
-            hist_data = self.historical_websocket.get_hist_tick_data(contract, query_time, start_time)
+            hist_data = self.historical_websocket.get_hist_tick_data(contract, end_time, start_time)
         else:
             bar_size = bar_size.replace(' ', '')
             if bar_size[-1] == 's':
                 bar_size = bar_size[:-1]
-            hist_data = self.historical_websocket.get_hist_bar_data(contract, query_time, start_time, bar_size)
+            hist_data = self.historical_websocket.get_hist_bar_data(contract, end_time, start_time, bar_size)
         DEFAULT_HISTORIC_DATA_ID = DEFAULT_HISTORIC_DATA_ID + 1
         self.hist_data[ticker_id] = hist_data
         return hist_data
@@ -451,3 +435,6 @@ class TD:
             for req_id in self.symbol_mkt_id_map[contract.upper()]:
                 del self.live_data[req_id]
             del self.symbol_mkt_id_map[contract.upper()]
+
+    def get_touchline(self):
+        self.live_websocket.send(f'{{"method": "touchline"}}')
